Type: Package
Package: teal.code
Title: Code Storage and Execution Class for 'teal' Applications
Version: 0.7.2
Date: 2026-06-26
Authors@R: c(
    person("Dawid", "Kaledkowski", , "dawid.kaledkowski@roche.com", role = "aut"),
    person("Aleksander", "Chlebowski", , "aleksander.chlebowski@contractors.roche.com", role = "aut"),
    person("Marcin", "Kosinski", , "marcin.kosinski.mk1@roche.com", role = "aut"),
    person("Pawel", "Rucki", , "pawel.rucki@roche.com", role = "aut"),
    person("Nikolas", "Burkoff", , "nikolas.burkoff@roche.com", role = "aut"),
    person("Mahmoud", "Hallal", , "mahmoud.hallal@roche.com", role = "aut"),
    person("Maciej", "Nasinski", , "maciej.nasinski@contractors.roche.com", role = "aut"),
    person("Konrad", "Pagacz", , "konrad.pagacz@contractors.roche.com", role = "aut"),
    person("Junlue", "Zhao", , "zhaoj88@gene.com", role = "aut"),
    person("Chendi", "Liao", , "chendi.liao@roche.com", role = "rev"),
    person("Dony", "Unardi", , "unardid@gene.com", role = c("rev", "cre")),
    person("F. Hoffmann-La Roche AG", role = c("cph", "fnd"))
  )
Description: Introduction of 'qenv' S4 class, that facilitates code
    execution and reproducibility in 'teal' applications.
License: Apache License 2.0
URL: https://insightsengineering.github.io/teal.code/,
        https://github.com/insightsengineering/teal.code
BugReports: https://github.com/insightsengineering/teal.code/issues
Depends: methods, R (>= 4.0)
Imports: checkmate (>= 2.1.0), cli (>= 3.4.0), evaluate (>= 1.0.0),
        grDevices, lifecycle (>= 0.2.0), rlang (>= 1.1.0), stats, utils
Suggests: knitr (>= 1.42), rmarkdown (>= 2.23), shiny (>= 1.6.0),
        testthat (>= 3.1.8), withr (>= 2.0.0)
VignetteBuilder: knitr, rmarkdown
RdMacros: lifecycle
Config/Needs/verdepcheck: mllg/checkmate, r-lib/cli, r-lib/lifecycle,
        r-lib/rlang, r-lib/cli, yihui/knitr, rstudio/rmarkdown,
        rstudio/shiny, r-lib/testthat, r-lib/withr
Config/Needs/website: insightsengineering/nesttemplate
Encoding: UTF-8
Language: en-US
Collate: 'qenv-c.R' 'qenv-class.R' 'qenv-errors.R' 'qenv-concat.R'
        'qenv-constructor.R' 'qenv-eval_code.R' 'qenv-extract.R'
        'qenv-get_code.R' 'qenv-get_env.R' 'qenv-get_messages.r'
        'qenv-get_outputs.R' 'qenv-get_var.R' 'qenv-get_warnings.R'
        'qenv-join.R' 'qenv-length.R' 'qenv-show.R' 'qenv-within.R'
        'teal.code-package.R' 'utils-get_code_dependency.R' 'utils.R'
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-28 21:41:43 UTC; unardid
Author: Dawid Kaledkowski [aut],
  Aleksander Chlebowski [aut],
  Marcin Kosinski [aut],
  Pawel Rucki [aut],
  Nikolas Burkoff [aut],
  Mahmoud Hallal [aut],
  Maciej Nasinski [aut],
  Konrad Pagacz [aut],
  Junlue Zhao [aut],
  Chendi Liao [rev],
  Dony Unardi [rev, cre],
  F. Hoffmann-La Roche AG [cph, fnd]
Maintainer: Dony Unardi <unardid@gene.com>
Repository: CRAN
Date/Publication: 2026-06-29 07:50:02 UTC
