# Generated by roxygen2: do not edit by hand

S3method("$",qenv.error)
S3method("[",qenv)
S3method("[[",qenv.error)
S3method(as.list,qenv.error)
S3method(c,qenv)
S3method(c,qenv.error)
S3method(length,qenv)
S3method(length,qenv.error)
S3method(names,qenv.error)
S3method(within,qenv)
S3method(within,qenv.error)
export(concat)
export(dev_suppress)
export(eval_code)
export(get_code)
export(get_env)
export(get_messages)
export(get_outputs)
export(get_var)
export(get_warnings)
export(join)
export(qenv)
exportClasses(qenv)
exportMethods(show)
importFrom(lifecycle,badge)
importFrom(methods,show)
