#' `teal.code`: Code storage and execution class for teal applications
#'
#' The `teal.code` package contains functionalities that can be used
#' to store and execute R calls. `qenv` class allows to create a `teal`
#' module which returns code to reproduce application outputs.
#'
#' @keywords internal
"_PACKAGE"

#' @importFrom lifecycle badge
NULL
